<?php
/**
 * Force SSL for Login, Admin, and Site pages.
 *
 * @package Flat101\Security\Model\Block
 */

namespace Flat101\Security\Model\Block;

use Carbon_Fields\Field\Field;
use Flat101\Security\Model\Entity\Block;
use Flat101\Security\Model\Entity\Setting;

/**
 * ForceSSLBlock class.
 */
class ForceSSLBlock extends Block {

	/**
	 * Block constructor.
	 */
	public function __construct() {
		parent::__construct(
			esc_html__( 'Force SSL', 'flat101-security' ),
			'force_ssl',
			array(
				new Setting( 'ssl_login', false, esc_html__( 'Force SSL for login page', 'flat101-security' ) ),
				new Setting( 'ssl_admin', false, esc_html__( 'Force SSL for admin pages', 'flat101-security' ) ),
				new Setting( 'ssl_site', false, esc_html__( 'Force SSL for all site', 'flat101-security' ) ),
			)
		);
	}

	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function get_description() {
		return wp_kses_post(
			__(
				'<p>Enabling the features provided in this section will enforce the usage of SSL (Secure Socket Layer) on the specified pages of your WordPress site. SSL is a standard security protocol which ensures that all data transmitted between the web server and browser remains encrypted, providing a secure channel for sensitive information to be transmitted over the web. <a href="https://wordpress.org/support/article/administration-over-ssl/" target="_blank">Learn more</a>.</p>

					<p><strong>1. Force SSL for Login Page:</strong>
					Enabling this feature will ensure that the login page of your website is accessed over HTTPS, thereby encrypting the login credentials being sent to the server, and protecting them from eavesdropping attacks.</p>

					<p><strong>2. Force SSL for Admin Pages:</strong>
					By activating this feature, all admin pages will be served over HTTPS. This is crucial for protecting the integrity and confidentiality of administrative interactions with your website.</p>

					<p><strong>3. Force SSL for Entire Site:</strong>
					This feature will enforce HTTPS across all pages of your site, ensuring that all data transmitted between users and your website is encrypted. This is highly recommended for enhancing the overall security posture of your website, and is favorable for SEO rankings as well.</p>',
				'flat101-security'
			)
		);
	}

	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function get_fields() {
		return array(
			Field::make( 'checkbox', $this->get_option_name( 'ssl_login' ), $this->get_setting_label( 'ssl_login' ) )
				->set_default_value( $this->get_setting_default_value( 'ssl_login' ) ),

			Field::make( 'checkbox', $this->get_option_name( 'ssl_admin' ), $this->get_setting_label( 'ssl_admin' ) )
				->set_default_value( $this->get_setting_default_value( 'ssl_admin' ) ),

			Field::make( 'checkbox', $this->get_option_name( 'ssl_site' ), $this->get_setting_label( 'ssl_site' ) )
				->set_default_value( $this->get_setting_default_value( 'ssl_site' ) ),
		);
	}

	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function run() {
		if (
			$this->get_setting_value( 'ssl_login' ) ||
			$this->get_setting_value( 'ssl_admin' ) ||
			$this->get_setting_value( 'ssl_site' )
		) {
			// $this->force_ssl();
		}
	}

	/**
	 * Force SSL on specified pages.
	 */
	public function force_ssl() {
		if ( $this->get_setting_value( 'ssl_login' ) && ! defined( 'FORCE_SSL_LOGIN' ) ) {
			define( 'FORCE_SSL_LOGIN', true );
		}

		if ( $this->get_setting_value( 'ssl_admin' ) && ! defined( 'FORCE_SSL_ADMIN' ) ) {
			define( 'FORCE_SSL_ADMIN', true );
		}

		if ( $this->get_setting_value( 'ssl_site' ) && ! is_ssl() ) {
			if ( 0 === strpos( $_SERVER['REQUEST_URI'], 'http' ) ) {
				wp_redirect( set_url_scheme( $_SERVER['REQUEST_URI'], 'https' ), 301 );
				exit();
			} else {
				wp_redirect( 'https://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'], 301 );
				exit();
			}
		}
	}
}
