<?php
/**
 * REST API.
 *
 * @package Flat101\Security\Model\Block
 */

namespace Flat101\Security\Model\Block;

use Carbon_Fields\Field\Field;
use Flat101\Security\Model\Entity\Block;
use Flat101\Security\Model\Entity\Setting;
use WP_Error;

/**
 * REST API
 */
class RestApiBlock extends Block {

	/**
	 * Block constructor.
	 */
	public function __construct() {
		parent::__construct(
			esc_html__( 'REST API', 'flat101-security' ),
			'rest_api',
			array(
				new Setting( 'disabled', false, esc_html__( 'Disable REST API', 'flat101-security' ) ),
				new Setting( 'user_logged_in', true, esc_html__( 'User logged in', 'flat101-security' ) ),
				new Setting( 'protect_users_endpoint', true, esc_html__( 'Protect users endpoint', 'flat101-security' ) ),
			)
		);
	}

	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function get_fields() {
			return array(

				Field::make( 'checkbox', $this->get_option_name( 'disabled' ), $this->get_setting_label( 'disabled' ) )
				->set_default_value( $this->get_setting_default_value( 'disabled' ) )
				->set_help_text(
					__(
						'WordPress uses the REST API. If you deactivate it, you must install and activate the Classic Editor plugin.',
						'flat101-security'
					)
				),

				Field::make( 'checkbox', $this->get_option_name( 'user_logged_in' ), $this->get_setting_label( 'user_logged_in' ) )
				->set_default_value( $this->get_setting_default_value( 'user_logged_in' ) )
				->set_help_text(
					__(
						'Only registered users will be able to use the endpoints.',
						'flat101-security'
					)
				)
				->set_conditional_logic(
					array(
						array(
							'field' => $this->get_option_name( 'disabled' ),
							'value' => false,
						),
					)
				),

				Field::make( 'checkbox', $this->get_option_name( 'protect_users_endpoint' ), $this->get_setting_label( 'protect_users_endpoint' ) )
				->set_default_value( $this->get_setting_default_value( 'protect_users_endpoint' ) )
				->set_help_text(
					__(
						'Unregistered users cannot access the list of users.',
						'flat101-security'
					)
				)
				->set_conditional_logic(
					array(
						array(
							'field' => $this->get_option_name( 'disabled' ),
							'value' => false,
						),
						array(
							'field' => $this->get_option_name( 'user_logged_in' ),
							'value' => false,
						),
					)
				),

			);
	}

	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function run() {
		if ( $this->get_setting_value( 'rest_api_disabled' ) ) {
			add_filter( 'rest_authentication_errors', array( $this, 'disabled' ) );
		} elseif ( $this->get_setting_value( 'user_logged_in' ) ) {
			add_filter( 'rest_authentication_errors', array( $this, 'force_user_logged_in' ) );
		} elseif ( $this->get_setting_value( 'protect_users_endpoint' ) ) {
			add_filter( 'rest_endpoints', array( $this, 'force_protect_users_endpoint' ) );
		}
	}

	/**
	 * Returns an error indicating that the REST API is disabled.
	 *
	 * @param WP_Error|null|true $errors WP_Error if authentication error, null if authentication method wasn't used,
	 *                                   true if authentication succeeded.
	 * @return WP_Error
	 */
	public function disabled( $errors ) {
			return new WP_Error(
				'rest_disabled',
				esc_html__( 'REST API disabled', 'flat101-security' ),
				array( 'status' => rest_authorization_required_code() )
			);
	}

	/**
	 * If the user is not logged in, it returns an error.
	 *
	 * @param WP_Error|null|true $errors WP_Error if authentication error, null if authentication method wasn't used,
	 *                                   true if authentication succeeded.
	 * @return WP_Error|null|true
	 */
	public function force_user_logged_in( $errors ) {
		if ( ! empty( $errors ) ) {
			return $errors;
		}

		if ( ! is_user_logged_in() ) {
			return new WP_Error(
				'rest_not_logged_in',
				esc_html__( 'You are not currently logged in.', 'flat101-security' ),
				array( 'status' => 401 )
			);
		}

		return $errors;
	}

	/**
	 * Disable REST API user endpoints for unlogged users.
	 *
	 * @param array $endpoints The available WP endpoints.
	 * @return array
	 */
	public function force_protect_users_endpoint( $endpoints ) {
		if ( ! is_user_logged_in() ) {
			if ( isset( $endpoints['/wp/v2/users'] ) ) {
				unset( $endpoints['/wp/v2/users'] );
			}

			if ( isset( $endpoints['/wp/v2/users/(?P<id>[\d]+)'] ) ) {
				unset( $endpoints['/wp/v2/users/(?P<id>[\d]+)'] );
			}
		}

		return $endpoints;
	}
}
